// @ myth - client - invent.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "client/invent.h"
#include "common/error.h"
#include "common/format.h"

// @@ INVENTORY_SYMBOL

// @@@ void INVENTORY_SYMBOL::AppendSymbol(PAGE* page, long* arg)

void INVENTORY_SYMBOL::AppendSymbol(PAGE* page, long* arg)
{
    PAGE::reset();

    while(PAGE::remain() > 0)
    {
	if(*arg > 0L)
	    (*arg)--;
	else
	    page->append(PAGE::current());

	PAGE::next();
    }
}

// @@ INVENTORY_BUNCH

// @@@ void INVENTORY_BUNCH::create(PAGE* page)

void INVENTORY_BUNCH::create(PAGE* page)
{
    for(int i = 0; i < SymbolCount; i++)
    {
	symbol[i].clear();
	symbol[i].letters = STRING("");
    }
    int current = 0;
    
    while(page->remain() > 0)
    {
	if(page->string()[0] == '\n')
	{
	    all[current] = page->string()[1];
	    page->next();
	    symbol[current].letters = page->current();
	    current++;
	}
	else
	    symbol[current - 1].append(page->current());

	page->next();
    }
    all[current] = 0;
}

// @@@ void INVENTORY_BUNCH::AppendPage(PAGE* page, 

void INVENTORY_BUNCH::AppendPage(PAGE* page, symbol_type type, long arg)
{
    for(int i = 0; all[i] != 0; i++)
	if(type == SYMBOL_BUNCH || all[i] == type)
	    symbol[i].AppendSymbol(page, &arg);
}

// @@@ INVENTORY_BUNCH::INVENTORY_BUNCH()

INVENTORY_BUNCH::INVENTORY_BUNCH()
{
    all[0] = 0;
}

// @@ INVENTORY

// @@@ void INVENTORY::CreatePage(inventory_type bunch,

void INVENTORY::CreatePage(inventory_type bunch, symbol_type symbol, long arg)
{
    page.clear();

    if(bunch == INVENTORY_ERROR)
	return;

    invent[bunch].AppendPage(&page, symbol, arg);

    if(page.length() < 1)
    {
	if(symbol == SYMBOL_BUNCH)
	    page.append("There's nothing in this bunch.");
	else
	    page.append(format("There's no %s in this bunch.",
			       SymbolName(symbol, PLURAL)));
    }
}    

// @@@ void INVENTORY::create(inventory_type bunch, PAGE* page)

void INVENTORY::create(inventory_type bunch, PAGE* page)
{
    invent[bunch].create(page);

    if(size() != Origo)
	NewBunch = bunch;
}

// @@@ void INVENTORY::show(inventory_type bunch,

void INVENTORY::show(inventory_type bunch, symbol_type symbol, long arg)
{
    CreatePage(bunch, symbol, arg);

    if(size() == Origo)
	CoverPage = TRUE;
    else
    {
	for(int y = 0; y < size().y; y++)
	{
	    if(page.remain() > 0)
	    {
		print(coor(0, y), page.string());
		page.next();
	    }
	    else
		print(coor(0, y), "");
	 }
    }
}

// @@@ void INVENTORY::update()
//
// Update inventory window

void INVENTORY::update()
{
    if(NewBunch != INVENTORY_ERROR)
    {
	show(NewBunch, SYMBOL_BUNCH, 0L);
	NewBunch = INVENTORY_ERROR;
    }
}

// @@@ const char* INVENTORY::symbol(inventory_type bunch)

const char* INVENTORY::symbol(inventory_type bunch)
{
    if(bunch == ERROR)
	return ALL_SYMBOLS;
    else if(invent[bunch].all[0] == 0)
	return "*";
    else
	return invent[bunch].all;
}

// @@@ const char* INVENTORY::letter(inventory_type bunch, symbol_type symbol)

const char* INVENTORY::letter(inventory_type bunch, symbol_type symbol)
{
    if(bunch == ERROR)
	return "A-Za-z";

    char* ptr = strchr(invent[bunch].all, symbol);

    if(ptr == NULL)
	return "*";
    
    int index = ptr - invent[bunch].all;
    
    if(invent[bunch].symbol[index].letters.string() == NULL)
	return "*";
    else
	return invent[bunch].symbol[index].letters.string();
}

// @@@ INVENTORY::INVENTORY()
//
// Create window with at least 40 columns, and 35 % of the remaining areal,
// iff there is at least 90 columns left.

INVENTORY::INVENTORY() : (WINDOW_RIGHT, 40, 35, 90)
{
    CoverPage = FALSE;
    NewBunch = INVENTORY_ERROR;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
