// @ myth - common - list.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// This file defines a generic list class.

// The "derived" class may provide a erase() function, which should
// work like remove(), but also delete the item.

// @@ begin

#ifndef LIST_HEADER
#define LIST_HEADER

// @@ include

#include "common/common.h"

// @@ struct item_link

struct item_link
{
    void* item;
    item_link* next;
    item_link(void* obj, item_link* link)
    { item = obj; next = link; }
};

// @@ class LIST

class LIST
{
    item_link* list;		// First item
    item_link* pointer;		// Current item
    item_link* last;		// Last item
    item_link* previous;	// Item before current item
    int links;			// #items
    int left;			// #items after current item
public:
    // Maintaining the list
    void add(void* item);	// Add item to the head of the list
    void insert(void* item);	// Insert item before current item
    void append(void* item);	// Append item to the tail of the list
    void remove();		// Remove the current item
    // Looping the list
    void reset(int target = 0);	// Point to first item
    inline void next()		// Point to next item
    {
	previous = pointer;
	pointer = pointer->next;
	left--;
    }
    inline void* current()	// Return current item
    { return (pointer == NULL) ? NULL : pointer->item; }
    // Misc
    void sort(pfEpVpV compare);	// Sort list using the compare function
    void clear();		// Remove all entries from the list
    inline int length()		// Number of items in the list
    { return links; }
    inline int remain()		// Number of items after the current one
    { return left; }
    inline int where()		// Position in list
    { return links - left; }
    LIST();
    ~LIST()
    { clear(); }
};

// @@ end

#endif LIST_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
