// @ myth - server - armor.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/armor.h"
#include "common/error.h"
#include "server/rnd.h"

// @@  LEATHER_ARMOR

// @@@ int LEATHER_ARMOR::protect(int power, damage_type* type)

int LEATHER_ARMOR::protect(int power, damage_type* type)
{
    // Protect against acid and stone
    *type &= ~(DAMAGE_ACID | DAMAGE_STONE);

    // Very good against cold
    if((*type & DAMAGE_COLD) && rnd(MyValue) > rnd(power))
	*type &= ~DAMAGE_COLD;

    // Usefull against lightening
    if((*type & DAMAGE_LIGHT) && rnd(MyValue) > rnd(power * 3))
	*type &= ~DAMAGE_LIGHT;

    // Not god against cutting weapons
    if((*type & DAMAGE_CUT) && rnd(MyValue) > rnd(power * 15))
	*type &= ~DAMAGE_CUT;

    // Much worse against stings
    if((*type & DAMAGE_STING) && rnd(MyValue) > rnd(power * 50))
	*type &= ~DAMAGE_STING;

    // Reasonable against normal hits
    if(*type == DAMAGE_HIT)
	return 750;		// 75 % for a *perfect* leather armor
    else
	return 100;		// 10 % of other kinds of damage
}

// @@  RING_MAIL

// @@@ int RING_MAIL::protect(int power, damage_type* type)

int RING_MAIL::protect(int power, damage_type* type)
{
    // Protect against stone
    if((*type & DAMAGE_STONE) && rnd.chance(MyValue, MyValue + 1))
	*type &= ~DAMAGE_STONE;

    // Damaged by acid!
    if((*type & DAMAGE_ACID) && !IsBlessed() && rnd.chance(2))
	damage();

    // Usefull against cutting weapons
    if((*type & DAMAGE_CUT) && rnd(MyValue) > rnd(power * 3))
	*type &= ~DAMAGE_CUT;

    // Better against normal hits
    if(*type == DAMAGE_HIT)
	return 850;		// 85 % for a *perfect* ring mail
    else
	return 200;		// 20 % of other kinds of damage
}

// @@  PLATE_MAIL

// @@@ int PLATE_MAIL::protect(int power, damage_type* type)

int PLATE_MAIL::protect(int power, damage_type* type)
{
    // Protect against stone
    *type &= ~DAMAGE_STONE;

    // Damaged by acid!
    if((*type & DAMAGE_ACID) && !IsBlessed())
	damage();

    // Very god against cutting weapons
    if((*type & DAMAGE_CUT) && rnd(MyValue) > rnd(power))
	*type &= ~DAMAGE_CUT;

    // Even better against stings
    if((*type & DAMAGE_STING) && rnd(MyValue * 2) > rnd(power))
	*type &= ~DAMAGE_STING;

    // Best against normal hits
    if(*type == DAMAGE_HIT)
	return 950;		// 95 % for a *perfect* plate mail
    else
	return 400;		// 40 % of other kinds of damage
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
