// @ myth - server - body.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef BODY_HEADER
#define BODY_HEADER

// @@ include

#include "common/common.h"
#include "server/watch.h"
#include "server/object.h"
#include "server/ability.h"

struct BINARY;
struct PARSE;
struct PLAYER;

// @@ enum body_type

enum body_type
{
    BODY_TYPE_HUMAN,
    BODY_TYPE_ACID_BLOB,
    BODY_TYPE_BAT,
    BODY_TYPE_RAT,
    BODY_TYPE_GOBLIN,
    BODY_TYPE_HOBGOBLIN,
    BODY_TYPE_ORC,
    BODY_TYPE_DRAGON,
    BODY_TYPE_LIZARD,
    BODY_TYPE_COCKATRICE,
    BODY_TYPE_WRAITH,
    BODY_TYPE_ELF,
    BODY_TYPE_TROLL,
    BODY_TYPE_FUNGUS,
    BODY_TYPE_END,
    BODY_TYPE_ERROR = ERROR
};

// @@ Body Weapons

// @@@ BODY_WEAPON

class BODY_WEAPON : public OBJECT
{
public:
    bool IsBodyPart()
    { return TRUE; }
};

// @@@ FIST

class FIST : public BODY_WEAPON
{
public:
    const char* name()
    { return "fist"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 2; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 20; }
    FIST() { }
};

extern FIST TheFist;

// @@@ WRAITH_FIST

class WRAITH_FIST : public BODY_WEAPON
{
public:
    const char* name()
    { return "fist"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 2; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 20; }
    damage_type DamageType()
    { return DAMAGE_DRAIN_LIFE | DAMAGE_HIT; }
    WRAITH_FIST() { }
};

extern WRAITH_FIST TheWraithFist;

// @@@ FUNGII

class FUNGII : public BODY_WEAPON
{
public:
    const char* name()
    { return "fungii"; }
    damage_type DamageType()
    { return DAMAGE_POISON; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 5; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 5; }
    FUNGII() { }
};

extern FUNGII TheFungii;

// @@@ TEETH

class TEETH : public BODY_WEAPON
{
public:
    const char* name()
    { return "teeth"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 10; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 1; }
    TEETH() { }
};

extern TEETH TheTeeth;

// @@@ COCK

class COCK : public BODY_WEAPON
{
public:
    const char* name()
    { return "teeth"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 4; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 2; }
    damage_type DamageType()
    { return DAMAGE_STONE | DAMAGE_HIT; }
    COCK() { }
};

extern COCK TheCock;

// @@@ TAIL

class TAIL : public BODY_WEAPON
{
public:
    const char* name()
    { return "tail"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 25; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 10; }
    TAIL() { }
};

extern TAIL TheTail;

// @@@ ACID

class ACID : public BODY_WEAPON
{
public:
    const char* name()
    { return "acid"; }
    int DamageBonus(MONSTER*, MONSTER*)
    { return 2; }
    int HitBonus(MONSTER*, MONSTER*)
    { return 3; }
    damage_type DamageType()
    { return DAMAGE_ACID | DAMAGE_HIT; }
    ACID() { }
};

extern ACID TheAcid;

// @@ BODY

class BODY
{
    // Health
    //
private:
    long when;			// When will it be tainted?
    long gone;			// When will it be removed?
    int health();		// How is the MaxHP / HP ratia?
protected:
    int experience;		// How experienced is it
    int HitPoint;		// Number of hit points left
    int MaxHitPoint;		// Number of HitPoint when in form
    virtual int constitution()	// How fast does HP regenerate, etc.
    { return 1; }
public:
    bool dead()			// Is monster dead?
    { return HitPoint < 1; }
    bool done()			// Dead and eaten up
    { return dead() && watch() > gone; }
    void kill();		// Monster is dead!
    void life()			// Bring back to life!
    { HitPoint = 1; }
    virtual cont hurt(int power, int type, OBJECT* weapon,
		      MONSTER* attack, MONSTER* defence, LEVEL* level);
    void SetExperience(int exp);
    void heal(int hp);		// Heal monster
    void RestoreHealth()
    { HitPoint = MaxHitPoint; }
    bool wounded()		// Is he wounded?
    { return HitPoint < MaxHitPoint; }
    //
    // Eat
    //
private:
    int FoodStatus()		// How full is his stomack?
    { return (10 * food) / stomach() + 1; }
    void eat(LEVEL* level, MONSTER* monster);
protected:
    long food;			// How much food in the stomack?
    virtual int FoodValue()	// Value when eaten
    { return weight(); }
    virtual bool poisonous()	// Is the corpse poisonous?
    { return FALSE; }
    virtual void effect(LEVEL*, MONSTER*);
    virtual QUESTION* DoEffect(LEVEL* level, PLAYER* player, CHANNEL*);
    virtual int stomach()
    { return 2 * size(); }
    virtual int digest()
    { return 1; }
    virtual bool CanChoke()
    { return TRUE; }
public:
    bool tainted()		// Is it to old?
    { return dead() && watch() > when; }
    void apply(LEVEL* level, MONSTER* monster);
    QUESTION* DoApply(LEVEL* level, PLAYER* player, CHANNEL* channel);
    cont feed(int value, force flag);
    virtual bool hungry()	// TRUE, iff the body needs food
    { return FoodStatus() < 5; }
    //
    // Strength
    //
protected:
    int MyStrength;		// Current strength
    int MaxStrength;		// Maximal strength
public:
    int strength();
    long carry()		// Carry Capacity
    { return max(1, 50L * sqr(strength())); }
    void AddStrength(int str);
    void RestoreStrength();
    void SetStrength(int str)
    { MyStrength = MaxStrength = str; }
    //
    // Abilities
    //
private:
    COUNTER_LIST count;
public:
    bool CheckAbility(ability_type ability)
    { return count.check(ability); }
    int GetValue(ability_type ability)
    { return count.value(ability); }
    void AddAbility(COUNTER* counter)
    { count.insert(counter); }
    void RemoveAbility(ability_type ability, force flag)
    { count.remove(ability, flag); }
    virtual bool CanMove(MONSTER*)
    { return TRUE; }
    virtual bool CanFight(MONSTER*)
    { return TRUE; }
    virtual void InsertAbility(MONSTER*) // Special actions when worn
    { /* Default: do nothing. */ }
    //
    // Identification
    //
protected:
    virtual char LivingLook()
    { return SYMBOL_ERROR; }
    virtual char DeadLook()
    { return SYMBOL_FOOD; }
    virtual int view()
    { return 2; }
    virtual int BodySpeed()
    { return 100; }
    virtual bool ColdBlood()	// Cold based monster
    { return FALSE; }
    virtual int armor()		// Protection from skin armor
    { return 0; }
public:
    virtual const char* BodyName();
    virtual body_type type()	// What kind of a body?
    { return BODY_TYPE_ERROR; }
    virtual object_type LookType() // What does it look like?
    { return OBJECT_ERROR; }
    char look()			// How does the monster look?
    { return dead() ? DeadLook() : LivingLook(); }
    virtual const char* name();
    virtual const char* PluralName(); // Name of several objects
    virtual const char* ClassName(); // Name of the kind of objects
    virtual const char* PluralClassName(); // Name several object of the kind
    virtual long size()
    { return 1; }
    virtual long weight()
    { return 1; }
    int speed();		// How fast does it move and fight?
    virtual OBJECT* weapon();	// What kind of damage does it do?
    int sight();		// How fare can it see?
    virtual int MaxRing()	// Number of ring fingers
    { return 0; }
    virtual int damage()	// General damage done byin fight
    { return 1; }
    //
    // Use
    //
public:
    virtual void hiss(LEVEL* level, MONSTER* monster);
    virtual void breathe(LEVEL* level, MONSTER* monster, coor);
    //
    // Administration
    //
public:
    void status(PLAYER* player);
    void DigestFood(MONSTER* monster, int val = 1);
    virtual void update(LEVEL* level, MONSTER* monster);
    virtual void save(BINARY* binary);
    BODY(BINARY* binary);
    BODY(int str);
};

// @@ extern

extern BODY* LoadBody(BINARY* binary);
extern BODY* CreateBody(int exp);

// @@ end

#endif BODY_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
