// @ myth - server - potion.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef POTION_HEADER
#define POTION_HEADER

// @@ include

#include "server/thing.h"

// @@ POTION_WATER

class POTION_WATER : public POTION
{
    // Identification
    //
protected:
    int BaseValue()
    { return 10; }
public:
    object_type type()
    { return POTION_OF_WATER; }
    const char* MagicName()
    { return "water"; }
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new POTION_WATER(this, arg); }
    POTION_WATER(BINARY* binary) : (binary) { }
    POTION_WATER(PARSE* parse) : (parse) { }
    POTION_WATER(POTION_WATER* thing,
			 long count) : (thing, count) { }
    POTION_WATER(long count = 1L) : (count) { }
};

// @@ POTION_STRENGTH

class POTION_STRENGTH : public POTION
{
    // Identification
    //
protected:
    int BaseValue()
    { return 150; }
public:
    object_type type()
    { return POTION_OF_STRENGTH; }
    const char* MagicName()
    { return "strength"; }
    //
    // Use
    //
    void effect(LEVEL*, MONSTER* monster);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new POTION_STRENGTH(this, arg); }
    POTION_STRENGTH(BINARY* binary) : (binary) { }
    POTION_STRENGTH(PARSE* parse) : (parse) { }
    POTION_STRENGTH(POTION_STRENGTH* thing,
			 long count) : (thing, count) { }
    POTION_STRENGTH(long count = 1L) : (count) { }
};

// @@ POTION_HEALING

class POTION_HEALING : public POTION
{
    // Identification
    //
protected:
    int BaseValue()
    { return 200; }
public:
    object_type type()
    { return POTION_OF_HEALING; }
    const char* MagicName()
    { return "healing"; }
    //
    // Use
    //
    void effect(LEVEL*, MONSTER*); // Special effect when applied
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new POTION_HEALING(this, arg); }
    POTION_HEALING(BINARY* binary) : (binary) { }
    POTION_HEALING(PARSE* parse) : (parse) { }
    POTION_HEALING(POTION_HEALING* thing, long count) : (thing, count) { }
    POTION_HEALING(long count = 1L) : (count) { }
};

// @@ POTION_EXPERIENCE

class POTION_EXPERIENCE : public POTION
{
    // Identification
    //
protected:
    int BaseValue()
    { return 500; }
public:
    object_type type()
    { return POTION_OF_EXPERIENCE; }
    const char* MagicName()
    { return "experience"; }
    //
    // Use
    //
    void effect(LEVEL*, MONSTER*); // Special effect when applied
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new POTION_EXPERIENCE(this, arg); }
    POTION_EXPERIENCE(BINARY* binary) : (binary) { }
    POTION_EXPERIENCE(PARSE* parse) : (parse) { }
    POTION_EXPERIENCE(POTION_EXPERIENCE* thing, long count) : (thing, count) { }
    POTION_EXPERIENCE(long count = 1L) : (count) { }
};

// @@ POTION_SIGHT

class POTION_SIGHT : public POTION
{
    // Identification
    //
protected:
    int BaseValue()
    { return 50; }
public:
    object_type type()
    { return POTION_OF_SIGHT; }
    const char* MagicName()
    { return "sight"; }
    //
    // Use
    //
    void effect(LEVEL*, MONSTER*); // Special effect when applied
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new POTION_SIGHT(this, arg); }
    POTION_SIGHT(BINARY* binary) : (binary) { }
    POTION_SIGHT(PARSE* parse) : (parse) { }
    POTION_SIGHT(POTION_SIGHT* thing, long count) : (thing, count) { }
    POTION_SIGHT(long count = 1L) : (count) { }
};

// @@ POTION_ENERGY

class POTION_ENERGY : public POTION
{
    // Identification
    //
protected:
    int BaseValue()
    { return 100; }
public:
    object_type type()
    { return POTION_OF_ENERGY; }
    const char* MagicName()
    { return "energy"; }
    //
    // Use
    //
    void effect(LEVEL* level, MONSTER* monster);
    //
    // Administration
    //
public:
    OBJECT* split(long arg)
    { MyCount -= arg; return new POTION_ENERGY(this, arg); }
    POTION_ENERGY(BINARY* binary) : (binary) { }
    POTION_ENERGY(PARSE* parse) : (parse) { }
    POTION_ENERGY(POTION_ENERGY* thing, long count) : (thing, count) { }
    POTION_ENERGY(long count = 1L) : (count) { }
};

// @@ end

#endif POTION_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
