// @ myth - server - print.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1990 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/player.h"
#include "common/error.h"
#include "server/rnd.h"
#include "common/format.h"

// @@ const char* PrintCount(long count)

const char* PrintCount(long count)
{
    static const char* const CountName[] = 
    {
	"none", "one", "two", "three", "four", "five", "six", "seven",
	"eight", "nine", "ten", "eleven", "a dozen",
    };
    
    if(count < 0)
    {
	bug.error("::PrintCount", "negative count");
	return "less than zero";
    }
    else if(count < 13)
	return CountName[count];
    else
	count += rnd(0, (int) (count / 13)) - rnd(0, (int) (count / 13));

    if(count < 19)
	return "more than a dozen";
    else if(count < 21)
	return "almost two dozen";
    else if(count < 30)
	return "two dozen";
    else if(count < 42)
	return "three dozen";
    else if(count < 80)
	return "several dozen";
    else if(count < 150)
	return "hundred";
    else if(count < 250)
	return "two hundred";
    else if(count < 700)
	return "several hundred";
    else if(count < 1500)
	return "thousand";
    else if(count < 8000)
	return "several thousand";
    else
	return "countless";
}

// @@ STRING PrintObject(OBJECT* object,

STRING PrintObject(OBJECT* object, print_flag flag,
	     PLAYER* player = NULL, long count = ERROR)
{
    if(count == ERROR)
	count = object->count();
    
    if(object == NULL)
    {
	bug.error("::PrintObject", "NULL object");
	return format("chaos");
    }
    char string[MessageSize];

    if(object->visible())
	string[0] = 0;
    else
    {
	(void) strcpy(string, "invisible ");

	if((flag & PRINT_FORCE) == 0 &&
	   (player == NULL || !player->SeeInvisible()))
	{
	    flag &= ~(PRINT_VALUE | PRINT_TYPE);
	}
    }
    if((flag & PRINT_FORCE) == 0)
    {
	// Player don't know the value
	if(player == NULL || !player->know(object))
	    flag &= ~PRINT_VALUE;
	// Player don't know this type (Monsters always know)
	if(!object->IsMonster() &&
	   (player == NULL || player->knowledge(object) < 1))
	{
	    flag &= ~PRINT_TYPE;
	}
    }
    STRING tmp_str = object->print(flag, count, player);
    const char* tmp_text = tmp_str.string();
    STRING name(strcat(string, tmp_text));

    if(flag & PRINT_ENTRY)
    {
	string[0] = object->letter();
	string[1] = object->symbol();
	string[2] = ' ';
	string[3] = 0;
    }
    else
	string[0] = 0;
    
    if(!object->IsUnique() || count != 1L || (flag & PRINT_TYPE) == 0)
    {
	if(flag & PRINT_THE)
	    (void) strcat(string, "the ");
	
	if((flag & PRINT_COUNT))
	{
	    if(count == 1L && !(flag & PRINT_THE))
	    {
		if(IsVowel(name()[0]))
		    (void) strcat(string, "an");
		else
		    (void) strcat(string, "a");
	    }
	    else
		(void) strcat(string, PrintCount(count));

	    (void) strcat(string, " ");
	}
    }
    (void) strcat(string, name());

    if((flag & PRINT_WORN) && player != NULL && player->pack.worn(object))
	(void) strcat(string, " (worn)");

    if((flag & PRINT_UPPER) && islower(string[0]))
	string[0] = toupper(string[0]);
	
    return format("%s", string); // Cheat the Garbage Collector.
    // Hey! There is no Garbage Collector.
    // Well, format can be used, if you want to keep a string for some time.
}

// @@ const char* print(OBJECT* object, print_flag flag,

const char* print(OBJECT* object, print_flag flag,	
		 PLAYER* player = NULL, long count = ERROR)
{ 
    STRING string = PrintObject(object, flag, player, count);
    const char* text = string.string();
    return text;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
